<?php
class Lottery_Picker {
    private $version;

    public function __construct() {
        $this->version = LOTTERY_PICKER_VERSION;
    }

    public function init() {
        // Add menu items
        add_action('admin_menu', array($this, 'add_admin_menu'));
        // Register scripts and styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_assets'));
        // Add AJAX handlers
        add_action('wp_ajax_process_csv_upload', array($this, 'process_csv_upload'));
        add_action('wp_ajax_run_lottery', array($this, 'run_lottery'));
        
        // Debug AJAX registration
        error_log('Lottery Picker: AJAX handlers registered');
    }

    public function add_admin_menu() {
        // Add main menu page
        add_menu_page(
            'Lottery Picker',
            'Lottery Picker',
            'manage_options',
            'lottery-picker',
            array($this, 'render_admin_page'),
            'dashicons-randomize',
            30
        );

        // Add Run Lottery as the first submenu item (this replaces the default submenu item)
        add_submenu_page(
            'lottery-picker',
            'Run Lottery',
            'Run Lottery',
            'manage_options',
            'lottery-picker',
            array($this, 'render_admin_page')
        );

        // Add Lottery Winners list
        global $submenu;
        add_submenu_page(
            'lottery-picker',
            'Lottery Winners',
            'Lottery Winners',
            'manage_options',
            'edit.php?post_type=lottery_winners'
        );

        // Fix the URL for the Lottery Winners page
        if (isset($submenu['lottery-picker'])) {
            foreach ($submenu['lottery-picker'] as $key => $item) {
                if ($item[2] === 'edit.php?post_type=lottery_winners') {
                    // Store the URL
                    $url = $item[2];
                    // Modify the URL to be absolute
                    $submenu['lottery-picker'][$key][2] = admin_url($url);
                }
            }
        }
    }

    public function enqueue_admin_assets($hook) {
        // Debug hook name
        error_log('Lottery Picker: Current admin page hook - ' . $hook);
        error_log('Lottery Picker: GET parameters - ' . print_r($_GET, true));

        // Check if we're on the winners list page
        $is_winners_list = (strpos($hook, 'edit.php') !== false && 
                           isset($_GET['post_type']) && 
                           $_GET['post_type'] === 'lottery_winners');

        // Check if we're on any other plugin pages
        $is_plugin_page = (
            $hook === 'toplevel_page_lottery-picker' ||
            $hook === 'lottery-picker_page_lottery-winners' ||
            $is_winners_list ||
            ($hook === 'post.php' && isset($_GET['post']) && get_post_type($_GET['post']) === 'lottery_winners')
        );

        error_log('Lottery Picker: Is winners list? ' . ($is_winners_list ? 'yes' : 'no'));
        error_log('Lottery Picker: Is plugin page? ' . ($is_plugin_page ? 'yes' : 'no'));

        if (!$is_plugin_page) {
            return;
        }

        // Always enqueue the admin CSS
        wp_enqueue_style(
            'lottery-picker-admin',
            LOTTERY_PICKER_PLUGIN_URL . 'assets/css/admin.css',
            array(),
            $this->version
        );

        // Enqueue lottery animation script only on the main lottery page
        if ($hook === 'toplevel_page_lottery-picker') {
            wp_enqueue_script(
                'lottery-animation',
                LOTTERY_PICKER_PLUGIN_URL . 'assets/js/lottery-animation.js',
                array('jquery'),
                $this->version,
                true
            );
        }

        // Enqueue share script only on the winners list page
        if ($is_winners_list) {
            wp_enqueue_script(
                'lottery-share',
                LOTTERY_PICKER_PLUGIN_URL . 'assets/js/lottery-share.js',
                array('jquery'),
                $this->version,
                true
            );

            wp_localize_script('lottery-share', 'lotteryPickerAdmin', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('lottery_picker_nonce'),
                'i18n' => array(
                    'copied' => __('Copied!', 'lottery-picker'),
                    'copyFailed' => __('Failed to copy', 'lottery-picker'),
                    'shareSuccess' => __('Share link copied to clipboard!', 'lottery-picker')
                )
            ));
        }

        // Main admin script for the lottery page
        if ($hook === 'toplevel_page_lottery-picker') {
            wp_enqueue_script(
                'lottery-picker-admin',
                LOTTERY_PICKER_PLUGIN_URL . 'assets/js/admin.js',
                array('jquery', 'lottery-animation'),
                $this->version,
                true
            );

            wp_localize_script('lottery-picker-admin', 'lotteryPickerAdmin', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'network_admin' => is_multisite() ? network_admin_url('admin-ajax.php') : '',
                'nonce' => wp_create_nonce('lottery_picker_nonce'),
                'is_multisite' => is_multisite()
            ));
        }
    }

    public function render_admin_page() {
        include LOTTERY_PICKER_PLUGIN_DIR . 'templates/admin-page.php';
    }

    public function render_winners_page() {
        include LOTTERY_PICKER_PLUGIN_DIR . 'templates/winners-page.php';
    }

    public function process_csv_upload() {
        error_log('Lottery Picker: process_csv_upload called');
        error_log('Lottery Picker: POST data - ' . print_r($_POST, true));
        error_log('Lottery Picker: FILES data - ' . print_r($_FILES, true));
        error_log('Lottery Picker: Is multisite - ' . (is_multisite() ? 'yes' : 'no'));
        error_log('Lottery Picker: Current blog ID - ' . get_current_blog_id());
        
        if (!check_ajax_referer('lottery_picker_nonce', 'nonce', false)) {
            error_log('Lottery Picker: Nonce verification failed');
            wp_send_json_error('Security check failed');
            return;
        }

        if (!current_user_can('manage_options')) {
            error_log('Lottery Picker: Unauthorized access attempt');
            wp_send_json_error('Unauthorized access');
            return;
        }

        $is_optional = isset($_POST['is_optional']) && $_POST['is_optional'] === '1';
        error_log('Lottery Picker: Processing ' . ($is_optional ? 'optional' : 'main') . ' CSV file');

        if (!isset($_FILES['csv_file'])) {
            error_log('Lottery Picker: No file uploaded - $_FILES array: ' . print_r($_FILES, true));
            wp_send_json_error('No file uploaded');
            return;
        }

        $file = $_FILES['csv_file'];
        error_log('Lottery Picker: File upload details - ' . print_r($file, true));
        
        // Check for upload errors
        if ($file['error'] !== UPLOAD_ERR_OK) {
            error_log('Lottery Picker: File upload error - ' . $file['error'] . ' - ' . $this->get_upload_error_message($file['error']));
            wp_send_json_error('File upload failed: ' . $this->get_upload_error_message($file['error']));
            return;
        }

        // Get file extension and check if it's CSV
        $file_extension = strtolower(pathinfo($file['name'], PATHINFO_EXTENSION));
        if ($file_extension !== 'csv') {
            error_log('Lottery Picker: Invalid file extension - ' . $file_extension);
            wp_send_json_error('Please upload a file with .csv extension');
            return;
        }

        // Verify it's a CSV file by checking content
        $csv_handle = fopen($file['tmp_name'], 'r');
        if ($csv_handle === false) {
            error_log('Lottery Picker: Failed to open CSV file - ' . $file['tmp_name']);
            wp_send_json_error('Unable to read file');
            return;
        }

        // Try to read the first line
        $first_line = fgets($csv_handle);
        if ($first_line === false) {
            fclose($csv_handle);
            error_log('Lottery Picker: Failed to read first line of file');
            wp_send_json_error('File appears to be empty');
            return;
        }

        // Reset file pointer
        rewind($csv_handle);

        // Get headers
        $headers = fgetcsv($csv_handle);
        error_log('Lottery Picker: CSV headers - ' . print_r($headers, true));
        
        if ($headers === false) {
            fclose($csv_handle);
            error_log('Lottery Picker: Failed to read CSV headers');
            wp_send_json_error('Unable to read CSV headers');
            return;
        }

        // Ensure we have at least one column
        if (empty($headers)) {
            fclose($csv_handle);
            error_log('Lottery Picker: CSV file has no columns');
            wp_send_json_error('CSV file has no columns');
            return;
        }

        fclose($csv_handle);

        // Get the upload directory for the current site in multisite
        if (is_multisite()) {
            switch_to_blog(get_current_blog_id());
            $upload_dir = wp_upload_dir();
            restore_current_blog();
        } else {
            $upload_dir = wp_upload_dir();
        }

        $temp_dir = $upload_dir['basedir'] . '/lottery-picker-temp';
        
        // Create temp directory if it doesn't exist
        if (!file_exists($temp_dir)) {
            wp_mkdir_p($temp_dir);
            
            // Create .htaccess to protect temp directory
            $htaccess = $temp_dir . '/.htaccess';
            if (!file_exists($htaccess)) {
                file_put_contents($htaccess, "deny from all\n");
            }

            // Create index.php to prevent directory listing
            $index_php = $temp_dir . '/index.php';
            if (!file_exists($index_php)) {
                file_put_contents($index_php, "<?php\n// Silence is golden.");
            }
        }

        // Generate unique filename with optional indicator and blog ID for multisite
        $prefix = $is_optional ? 'lottery_optional_' : 'lottery_';
        if (is_multisite()) {
            $prefix .= 'blog' . get_current_blog_id() . '_';
        }
        $temp_file = $temp_dir . '/' . uniqid($prefix, true) . '.csv';
        
        // Move uploaded file to our temp location
        if (!move_uploaded_file($file['tmp_name'], $temp_file)) {
            error_log('Lottery Picker: Failed to move uploaded file to - ' . $temp_file);
            wp_send_json_error('Failed to process uploaded file');
            return;
        }

        error_log('Lottery Picker: Successfully processed ' . ($is_optional ? 'optional' : 'main') . ' CSV file - ' . $temp_file);
        
        // Send success response with headers and temp file location
        wp_send_json_success(array(
            'columns' => $headers,
            'temp_file' => $temp_file,
            'is_optional' => $is_optional
        ));
    }

    private function get_upload_error_message($error_code) {
        switch ($error_code) {
            case UPLOAD_ERR_INI_SIZE:
                return 'The uploaded file exceeds the upload_max_filesize directive in php.ini';
            case UPLOAD_ERR_FORM_SIZE:
                return 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form';
            case UPLOAD_ERR_PARTIAL:
                return 'The uploaded file was only partially uploaded';
            case UPLOAD_ERR_NO_FILE:
                return 'No file was uploaded';
            case UPLOAD_ERR_NO_TMP_DIR:
                return 'Missing a temporary folder';
            case UPLOAD_ERR_CANT_WRITE:
                return 'Failed to write file to disk';
            case UPLOAD_ERR_EXTENSION:
                return 'A PHP extension stopped the file upload';
            default:
                return 'Unknown upload error';
        }
    }

    public function run_lottery() {
        check_ajax_referer('lottery_picker_nonce', 'nonce');

        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized access');
        }

        $lottery_name = sanitize_text_field($_POST['lottery_name']);
        $num_winners = intval($_POST['num_winners']);
        $selected_column = intval($_POST['selected_column']);
        $csv_file = $_POST['csv_file'];
        $previous_csv_file = isset($_POST['previous_csv_file']) ? $_POST['previous_csv_file'] : '';
        $previous_selected_column = isset($_POST['previous_selected_column']) ? intval($_POST['previous_selected_column']) : -1;

        if (!file_exists($csv_file)) {
            wp_send_json_error('CSV file not found');
        }

        // Track entries from each source
        $main_entries_count = 0;
        $previous_entries_count = 0;

        // Read main CSV and get entries
        $csv_handle = fopen($csv_file, 'r');
        $headers = fgetcsv($csv_handle);
        $all_rows = array();
        $entries = array();
        
        while (($data = fgetcsv($csv_handle)) !== false) {
            $all_rows[] = array('source' => 'main', 'data' => $data);
            if (isset($data[$selected_column])) {
                $entries[] = $data[$selected_column];
                $main_entries_count++;
            }
        }
        fclose($csv_handle);

        // Get previous non-winners if provided
        $previous_non_winners = array();
        if ($previous_csv_file && file_exists($previous_csv_file) && $previous_selected_column >= 0) {
            error_log('Lottery Picker: Processing previous non-winners CSV');
            $prev_handle = fopen($previous_csv_file, 'r');
            $prev_headers = fgetcsv($prev_handle);
            
            // If headers don't match, pad the shorter one with empty values
            $header_count = count($headers);
            $prev_header_count = count($prev_headers);
            if ($header_count > $prev_header_count) {
                $prev_headers = array_pad($prev_headers, $header_count, '');
            } elseif ($prev_header_count > $header_count) {
                $headers = array_pad($headers, $prev_header_count, '');
            }
            
            while (($data = fgetcsv($prev_handle)) !== false) {
                // Pad data array if needed
                if (count($data) < count($headers)) {
                    $data = array_pad($data, count($headers), '');
                }
                $all_rows[] = array('source' => 'previous', 'data' => $data);
                if (isset($data[$previous_selected_column])) {
                    $previous_non_winners[] = $data[$previous_selected_column];
                    $entries[] = $data[$previous_selected_column];
                    $previous_entries_count++;
                }
            }
            fclose($prev_handle);
            error_log('Lottery Picker: Previous entries count: ' . $previous_entries_count);
        }

        $total_entries = $main_entries_count + $previous_entries_count;
        error_log('Lottery Picker: Total entries: ' . $total_entries . ' (Main: ' . $main_entries_count . ', Previous: ' . $previous_entries_count . ')');

        // Select winners with weighted probability
        $winners = $this->select_winners($entries, $num_winners, $previous_non_winners);

        // Create non-winners CSV
        $upload_dir = wp_upload_dir();
        $non_winners_dir = $upload_dir['basedir'] . '/lottery-non-winners';
        if (!file_exists($non_winners_dir)) {
            wp_mkdir_p($non_winners_dir);
        }

        $non_winners_file = $non_winners_dir . '/' . sanitize_file_name($lottery_name . '-non-winners-' . date('Y-m-d-H-i-s') . '.csv');
        $non_winners_handle = fopen($non_winners_file, 'w');

        // Write headers
        fputcsv($non_winners_handle, $headers);

        // Write non-winning rows from both files
        foreach ($all_rows as $row) {
            $data = $row['data'];
            $column_index = ($row['source'] === 'main') ? $selected_column : $previous_selected_column;
            
            if (isset($data[$column_index]) && !in_array($data[$column_index], $winners)) {
                fputcsv($non_winners_handle, $data);
            }
        }
        fclose($non_winners_handle);

        // Get current date and time
        $draw_date = current_time('mysql');

        // Create post with minimal content
        $post_id = wp_insert_post(array(
            'post_title' => $lottery_name . ' - Winners',
            'post_content' => '',
            'post_type' => 'lottery_winners',
            'post_status' => 'publish'
        ));

        if (is_wp_error($post_id)) {
            wp_send_json_error('Failed to create winners post');
            return;
        }

        // Save meta data
        update_post_meta($post_id, '_lottery_name', $lottery_name);
        update_post_meta($post_id, '_total_entries', $total_entries);
        update_post_meta($post_id, '_main_entries', $main_entries_count);
        update_post_meta($post_id, '_previous_entries', $previous_entries_count);
        update_post_meta($post_id, '_draw_date', $draw_date);
        update_post_meta($post_id, '_winners', $winners);
        update_post_meta($post_id, '_non_winners_csv', str_replace($upload_dir['basedir'], $upload_dir['baseurl'], $non_winners_file));

        // Clean up temporary files
        if (file_exists($csv_file)) {
            unlink($csv_file);
        }
        if ($previous_csv_file && file_exists($previous_csv_file)) {
            unlink($previous_csv_file);
        }

        wp_send_json_success(array(
            'winners' => $winners,
            'total_entries' => $total_entries,
            'main_entries' => $main_entries_count,
            'previous_entries' => $previous_entries_count,
            'post_id' => $post_id,
            'draw_date' => $draw_date,
            'non_winners_csv' => str_replace($upload_dir['basedir'], $upload_dir['baseurl'], $non_winners_file)
        ));
    }

    private function select_winners($entries, $num_winners, $previous_non_winners = array()) {
        $total_entries = count($entries);
        $num_winners = min($num_winners, $total_entries);
        $winners = array();
        
        // Create weighted entries array
        $weighted_entries = array();
        foreach ($entries as $index => $entry) {
            // Add entry once by default
            $weighted_entries[] = $entry;
            
            // If entry is in previous non-winners, add it two more times (30% higher chance)
            if (in_array($entry, $previous_non_winners)) {
                $weighted_entries[] = $entry;
                $weighted_entries[] = $entry;
            }
        }
        
        // Randomly select winners from weighted array
        $selected_indices = array();
        while (count($winners) < $num_winners) {
            $index = array_rand($weighted_entries);
            $winner = $weighted_entries[$index];
            
            // Only add if not already selected
            if (!in_array($winner, $winners)) {
                $winners[] = $winner;
            }
            
            // Remove all instances of this winner from weighted entries
            $weighted_entries = array_filter($weighted_entries, function($entry) use ($winner) {
                return $entry !== $winner;
            });
            
            // Reindex array
            $weighted_entries = array_values($weighted_entries);
            
            // Break if no more entries available
            if (empty($weighted_entries)) {
                break;
            }
        }
        
        return $winners;
    }
} 