<?php
class Lottery_Winner_Post_Type {
    public function __construct() {
        add_action('init', array($this, 'register_post_type'));
        add_action('admin_init', array($this, 'add_meta_boxes'));
        add_action('save_post', array($this, 'save_meta_box_data'));
        add_filter('the_content', array($this, 'display_lottery_details'));
        add_action('admin_head', array($this, 'hide_editor'));
    }

    public function register_post_type() {
        $labels = array(
            'name'               => _x('Lottery Winners', 'post type general name', 'lottery-picker'),
            'singular_name'      => _x('Lottery Winner', 'post type singular name', 'lottery-picker'),
            'menu_name'          => _x('Lottery Winners', 'admin menu', 'lottery-picker'),
            'name_admin_bar'     => _x('Lottery Winner', 'add new on admin bar', 'lottery-picker'),
            'add_new'            => _x('Add New', 'lottery winner', 'lottery-picker'),
            'add_new_item'       => __('Add New Lottery Winner', 'lottery-picker'),
            'new_item'           => __('New Lottery Winner', 'lottery-picker'),
            'edit_item'          => __('Edit Lottery Winner', 'lottery-picker'),
            'view_item'          => __('View Lottery Winner', 'lottery-picker'),
            'all_items'          => __('All Winners', 'lottery-picker'),
            'search_items'       => __('Search Winners', 'lottery-picker'),
            'not_found'          => __('No lottery winners found.', 'lottery-picker'),
            'not_found_in_trash' => __('No lottery winners found in Trash.', 'lottery-picker')
        );

        $args = array(
            'labels'              => $labels,
            'public'              => true,
            'publicly_queryable'  => true,
            'show_ui'            => true,
            'show_in_menu'       => false,
            'show_in_admin_bar'  => false,
            'menu_position'      => null,
            'query_var'          => true,
            'rewrite'            => array(
                'slug'       => 'lottery-winners',
                'with_front' => true,
                'feeds'      => true,
                'pages'      => true
            ),
            'capability_type'    => 'post',
            'has_archive'        => true,
            'hierarchical'       => false,
            'supports'           => array('title'),
            'map_meta_cap'       => true,
            'delete_with_user'   => false,
            'can_export'         => true
        );

        register_post_type('lottery_winners', $args);

        // Register custom permalink structure
        add_filter('post_type_link', array($this, 'lottery_winner_permalink'), 10, 2);
        
        // Add filter to modify admin body class
        add_filter('admin_body_class', array($this, 'modify_admin_body_class'));
    }

    public function lottery_winner_permalink($post_link, $post) {
        if ($post->post_type === 'lottery_winners' && $post->post_status !== 'auto-draft') {
            $lottery_name = get_post_meta($post->ID, '_lottery_name', true);
            if ($lottery_name) {
                $lottery_slug = sanitize_title($lottery_name);
                $post_link = str_replace('%lottery_winners%', $lottery_slug, $post_link);
            }
        }
        return $post_link;
    }

    public function modify_admin_body_class($classes) {
        if (is_admin() && isset($_GET['post_type']) && $_GET['post_type'] === 'lottery_winners') {
            $classes .= ' lottery-picker-admin';
        }
        return $classes;
    }

    // Hide the editor on lottery winners post type
    public function hide_editor() {
        if (get_post_type() === 'lottery_winners') {
            echo '<style type="text/css">';
            echo '#postdivrich { display: none; }';
            echo '</style>';
        }
    }

    public function add_meta_boxes() {
        add_meta_box(
            'lottery_winner_details',
            __('Lottery Details', 'lottery-picker'),
            array($this, 'render_meta_box'),
            'lottery_winners',
            'normal',
            'high'
        );
    }

    public function render_meta_box($post) {
        // Add nonce for security
        wp_nonce_field('lottery_winner_meta_box', 'lottery_winner_meta_box_nonce');

        // Get existing values
        $lottery_name = get_post_meta($post->ID, '_lottery_name', true);
        $total_entries = get_post_meta($post->ID, '_total_entries', true);
        $draw_date = get_post_meta($post->ID, '_draw_date', true);
        $winners = get_post_meta($post->ID, '_winners', true);
        $non_winners_csv = get_post_meta($post->ID, '_non_winners_csv', true);

        ?>
        <style type="text/css">
            .lottery-winner-details .form-field {
                margin: 1em 0;
                padding: 10px;
                background: #f9f9f9;
                border: 1px solid #e5e5e5;
                border-radius: 4px;
            }
            .lottery-winner-details label {
                display: block;
                font-weight: 600;
                margin-bottom: 5px;
            }
            .lottery-winner-details .readonly-field {
                background: #fff;
                padding: 8px;
                border: 1px solid #ddd;
                border-radius: 4px;
            }
            .lottery-winner-details .winners-list {
                margin-top: 1em;
            }
            .lottery-winner-details .winners-list ol {
                margin-left: 20px;
            }
            .lottery-winner-details .download-section {
                margin-top: 1em;
                padding: 10px;
                background: #f0f6fc;
                border: 1px solid #2271b1;
                border-radius: 4px;
            }
            .lottery-winner-details .download-section .button {
                display: inline-flex;
                align-items: center;
                gap: 5px;
            }
            .lottery-winner-details .download-section .dashicons {
                width: 16px;
                height: 16px;
                font-size: 16px;
            }
        </style>
        <div class="lottery-winner-details">
            <div class="form-field">
                <label><?php _e('Lottery Name:', 'lottery-picker'); ?></label>
                <div class="readonly-field"><?php echo esc_html($lottery_name); ?></div>
            </div>
            
            <div class="form-field">
                <label><?php _e('Total Entries:', 'lottery-picker'); ?></label>
                <div class="readonly-field"><?php echo esc_html($total_entries); ?></div>
            </div>
            
            <div class="form-field">
                <label><?php _e('Draw Date:', 'lottery-picker'); ?></label>
                <div class="readonly-field"><?php echo esc_html(get_date_from_gmt($draw_date, 'F j, Y g:i a')); ?></div>
            </div>

            <?php if (is_array($winners) && !empty($winners)): ?>
            <div class="form-field winners-list">
                <label><?php _e('Winners:', 'lottery-picker'); ?></label>
                <div class="readonly-field">
                    <ol>
                        <?php foreach ($winners as $winner): ?>
                            <li><?php echo esc_html($winner); ?></li>
                        <?php endforeach; ?>
                    </ol>
                </div>
            </div>
            <?php endif; ?>

            <?php if ($non_winners_csv): ?>
            <div class="download-section">
                <label><?php _e('Download Options:', 'lottery-picker'); ?></label>
                <p>
                    <a href="<?php echo esc_url($non_winners_csv); ?>" class="button button-secondary" download>
                        <span class="dashicons dashicons-download"></span>
                        <?php _e('Download Non-Winners CSV', 'lottery-picker'); ?>
                    </a>
                </p>
                <p class="description">
                    <?php _e('Download the CSV file containing all entries except the winners.', 'lottery-picker'); ?>
                </p>
            </div>
            <?php endif; ?>
        </div>
        <?php
    }

    public function display_lottery_details($content) {
        // Only modify content for our custom post type on the frontend
        if (!is_singular('lottery_winners') || !in_the_loop() || !is_main_query()) {
            return $content;
        }

        $post_id = get_the_ID();
        $lottery_name = get_post_meta($post_id, '_lottery_name', true);
        $total_entries = get_post_meta($post_id, '_total_entries', true);
        $draw_date = get_post_meta($post_id, '_draw_date', true);
        $winners = get_post_meta($post_id, '_winners', true);

        ob_start();
        ?>
        <div class="lottery-winner-details">
            <div class="lottery-info">
                <p><strong><?php _e('Lottery Name:', 'lottery-picker'); ?></strong> <?php echo esc_html($lottery_name); ?></p>
                <p><strong><?php _e('Total Entries:', 'lottery-picker'); ?></strong> <?php echo esc_html($total_entries); ?></p>
                <p><strong><?php _e('Draw Date:', 'lottery-picker'); ?></strong> <?php echo esc_html(get_date_from_gmt($draw_date, 'F j, Y g:i a')); ?></p>
            </div>

            <?php if (is_array($winners) && !empty($winners)): ?>
            <div class="winners-list">
                <h3><?php _e('Winners:', 'lottery-picker'); ?></h3>
                <ol>
                    <?php foreach ($winners as $winner): ?>
                        <li><?php echo esc_html($winner); ?></li>
                    <?php endforeach; ?>
                </ol>
            </div>
            <?php endif; ?>
        </div>
        <?php
        $lottery_content = ob_get_clean();
        return $lottery_content;
    }

    public function save_meta_box_data($post_id) {
        // We don't save anything here as all data is read-only
        return;
    }
} 