<?php
/**
 * Plugin Name: Lottery Picker
 * Plugin URI: https://genexmarketing.com
 * Description: A plugin to manage lottery draws using CSV data with animated winner selection.
 * Version: 1.0.0
 * Author: Genex Marketing Agency Ltd.
 * Author URI: https://genexmarketing.com
 * Text Domain: lottery-picker
 * License: GPL v2 or later
 */

if (!defined('ABSPATH')) {
    exit;
}

// Define plugin constants
define('LOTTERY_PICKER_VERSION', '1.0.0');
define('LOTTERY_PICKER_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('LOTTERY_PICKER_PLUGIN_URL', plugin_dir_url(__FILE__));

// Include required files
require_once LOTTERY_PICKER_PLUGIN_DIR . 'includes/class-lottery-picker.php';
require_once LOTTERY_PICKER_PLUGIN_DIR . 'includes/class-lottery-winner-post-type.php';

// Initialize the plugin
function lottery_picker_init() {
    // Initialize post type
    $lottery_winner_post_type = new Lottery_Winner_Post_Type();
    
    // Initialize main plugin
    $lottery_picker = new Lottery_Picker();
    $lottery_picker->init();
}
add_action('plugins_loaded', 'lottery_picker_init');

// Activation hook
register_activation_hook(__FILE__, 'lottery_picker_activate');
function lottery_picker_activate() {
    // Initialize post type
    $lottery_winner_post_type = new Lottery_Winner_Post_Type();
    
    // Register post type
    $lottery_winner_post_type->register_post_type();
    
    // Clear the permalinks after the post type has been registered
    flush_rewrite_rules();
    
    // Set up default options
    add_option('lottery_picker_settings', array(
        'version' => LOTTERY_PICKER_VERSION
    ));
}

// Deactivation hook
register_deactivation_hook(__FILE__, 'lottery_picker_deactivate');
function lottery_picker_deactivate() {
    // Clear the permalinks to remove our post type's rules
    flush_rewrite_rules();
}

// Add rewrite flush on plugin update
function lottery_picker_version_check() {
    $current_version = get_option('lottery_picker_settings')['version'];
    if ($current_version !== LOTTERY_PICKER_VERSION) {
        flush_rewrite_rules();
        update_option('lottery_picker_settings', array(
            'version' => LOTTERY_PICKER_VERSION
        ));
    }
}
add_action('init', 'lottery_picker_version_check', 5); 