<?php
if (!defined('ABSPATH')) {
    exit;
}

$paged = get_query_var('paged') ? get_query_var('paged') : 1;
$args = array(
    'post_type' => 'lottery_winners',
    'posts_per_page' => 10,
    'paged' => $paged,
    'orderby' => 'date',
    'order' => 'DESC'
);

$winners_query = new WP_Query($args);
?>

<div class="wrap">
    <h1><?php echo esc_html(get_admin_page_title()); ?></h1>

    <?php if ($winners_query->have_posts()) : ?>
        <table class="wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <th><?php _e('Lottery', 'lottery-picker'); ?></th>
                    <th><?php _e('Draw Date', 'lottery-picker'); ?></th>
                    <th><?php _e('Total Entries', 'lottery-picker'); ?></th>
                    <th><?php _e('Actions', 'lottery-picker'); ?></th>
                </tr>
            </thead>
            <tbody>
                <?php while ($winners_query->have_posts()) : $winners_query->the_post(); ?>
                    <tr>
                        <td>
                            <?php echo esc_html(get_post_meta(get_the_ID(), '_lottery_name', true)); ?>
                        </td>
                        <td>
                            <?php echo esc_html(get_date_from_gmt(get_post_meta(get_the_ID(), '_draw_date', true), 'F j, Y g:i a')); ?>
                        </td>
                        <td>
                            <?php echo esc_html(get_post_meta(get_the_ID(), '_total_entries', true)); ?>
                        </td>
                        <td>
                            <a href="<?php echo get_edit_post_link(); ?>" class="button button-small">
                                <?php _e('Edit', 'lottery-picker'); ?>
                            </a>
                            <a href="<?php echo get_permalink(); ?>" class="button button-small" target="_blank">
                                <?php _e('View', 'lottery-picker'); ?>
                            </a>
                            <?php 
                            $non_winners_csv = get_post_meta(get_the_ID(), '_non_winners_csv', true);
                            if ($non_winners_csv): 
                            ?>
                            <a href="<?php echo esc_url($non_winners_csv); ?>" class="button button-small" download>
                                <span class="dashicons dashicons-download"></span>
                                <?php _e('Non-Winners', 'lottery-picker'); ?>
                            </a>
                            <?php endif; ?>
                            <button class="button button-small share-lottery" 
                                    data-url="<?php echo esc_url(get_permalink()); ?>"
                                    data-lottery="<?php echo esc_attr(get_post_meta(get_the_ID(), '_lottery_name', true)); ?>">
                                <?php _e('Share', 'lottery-picker'); ?>
                            </button>
                            <?php
                            $delete_url = wp_nonce_url(
                                admin_url('post.php?post=' . get_the_ID() . '&action=delete'),
                                'delete-post_' . get_the_ID()
                            );
                            ?>
                            <a href="<?php echo $delete_url; ?>" class="button button-small button-link-delete" 
                               onclick="return confirm('<?php _e('Are you sure you want to delete this lottery result?', 'lottery-picker'); ?>');">
                                <?php _e('Delete', 'lottery-picker'); ?>
                            </a>
                        </td>
                    </tr>
                <?php endwhile; ?>
            </tbody>
        </table>

        <?php
        echo '<div class="tablenav bottom">';
        echo '<div class="tablenav-pages">';
        echo paginate_links(array(
            'base' => add_query_arg('paged', '%#%'),
            'format' => '',
            'prev_text' => __('&laquo;'),
            'next_text' => __('&raquo;'),
            'total' => $winners_query->max_num_pages,
            'current' => $paged
        ));
        echo '</div>';
        echo '</div>';
        ?>

    <?php else : ?>
        <p><?php _e('No lottery winners found.', 'lottery-picker'); ?></p>
    <?php endif; ?>

    <?php wp_reset_postdata(); ?>
</div> 